@@ -16,8 +16,7 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free
- * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * License along with this library.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <glibmm.h>
@@ -49,20 +48,26 @@
 #include <giomm/dbuserror.h>
 #include <giomm/dbuserrorutils.h>
 #include <giomm/dbusinterface.h>
+#include <giomm/dbusinterfaceskeleton.h>
 #include <giomm/dbusinterfacevtable.h>
 #include <giomm/dbusintrospection.h>
 #include <giomm/dbusmenumodel.h>
 #include <giomm/dbusmessage.h>
 #include <giomm/dbusmethodinvocation.h>
 #include <giomm/dbusobject.h>
+#include <giomm/dbusobjectmanager.h>
+#include <giomm/dbusobjectmanagerclient.h>
+#include <giomm/dbusobjectmanagerserver.h>
+#include <giomm/dbusobjectproxy.h>
+#include <giomm/dbusobjectskeleton.h>
 #include <giomm/dbusownname.h>
 #include <giomm/dbusproxy.h>
 #include <giomm/dbusserver.h>
 #include <giomm/dbussubtreevtable.h>
 #include <giomm/dbusutils.h>
 #include <giomm/dbuswatchname.h>
-#ifndef G_OS_WIN32
-# include <giomm/desktopappinfo.h>
+#if !defined(G_OS_WIN32) && !defined(GLIBMM_OS_COCOA)
+#include <giomm/desktopappinfo.h>
 #endif
 #include <giomm/drive.h>
 #include <giomm/emblem.h>
@@ -89,6 +94,8 @@
 #include <giomm/initable.h>
 #include <giomm/inputstream.h>
 #include <giomm/iostream.h>
+#include <giomm/listmodel.h>
+#include <giomm/liststore.h>
 #include <giomm/loadableicon.h>
 #include <giomm/memoryinputstream.h>
 #include <giomm/memoryoutputstream.h>
@@ -100,8 +107,11 @@
 #include <giomm/mount.h>
 #include <giomm/mountoperation.h>
 #include <giomm/networkaddress.h>
+#include <giomm/networkmonitor.h>
 #include <giomm/networkservice.h>
+#include <giomm/notification.h>
 #include <giomm/outputstream.h>
+#include <giomm/permission.h>
 #include <giomm/pollableinputstream.h>
 #include <giomm/pollableoutputstream.h>
 #include <giomm/proxy.h>
@@ -112,8 +122,13 @@
 #include <giomm/resource.h>
 #include <giomm/seekable.h>
 #include <giomm/settings.h>
+#include <giomm/settingsschema.h>
+#include <giomm/settingsschemakey.h>
+#include <giomm/settingsschemasource.h>
 #include <giomm/simpleaction.h>
 #include <giomm/simpleactiongroup.h>
+#include <giomm/simpleiostream.h>
+#include <giomm/simplepermission.h>
 #include <giomm/socket.h>
 #include <giomm/socketaddress.h>
 #include <giomm/socketaddressenumerator.h>
@@ -123,15 +138,12 @@
 #include <giomm/socketcontrolmessage.h>
 #include <giomm/socketlistener.h>
 #include <giomm/socketservice.h>
+#include <giomm/socketsource.h>
 #include <giomm/srvtarget.h>
 #include <giomm/tcpconnection.h>
+#include <giomm/tcpwrapperconnection.h>
 #include <giomm/themedicon.h>
 #include <giomm/threadedsocketservice.h>
-
-//TODO: Re-add these includes after the next stable release when the
-//development of these classes can continue without risking needing an API or
-//ABI break later.
-#if 0
 #include <giomm/tlscertificate.h>
 #include <giomm/tlsclientconnection.h>
 #include <giomm/tlsconnection.h>
@@ -139,16 +151,14 @@
 #include <giomm/tlsinteraction.h>
 #include <giomm/tlspassword.h>
 #include <giomm/tlsserverconnection.h>
-#endif
-
 #ifndef G_OS_WIN32
-# include <giomm/unixconnection.h>
-# include <giomm/unixcredentialsmessage.h>
-# include <giomm/unixfdlist.h>
-# include <giomm/unixfdmessage.h>
-# include <giomm/unixinputstream.h>
-# include <giomm/unixoutputstream.h>
-# include <giomm/unixsocketaddress.h>
+#include <giomm/unixconnection.h>
+#include <giomm/unixcredentialsmessage.h>
+#include <giomm/unixfdlist.h>
+#include <giomm/unixfdmessage.h>
+#include <giomm/unixinputstream.h>
+#include <giomm/unixoutputstream.h>
+#include <giomm/unixsocketaddress.h>
 #endif
 #include <giomm/volume.h>
 #include <giomm/volumemonitor.h>
