@@ -13,8 +13,7 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free
- * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * License along with this library.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef _GLIBMM_H
@@ -27,13 +26,14 @@
  * glibmm is the official C++ interface for the popular cross-platform library %Glib.
  * It provides non-UI API that is not available in standard C++ and makes it
  * possible for gtkmm to wrap GObject-based APIs.
- * See also the <a href="http://library.gnome.org/devel/gtkmm-tutorial/stable/">
+ * See also the <a href="https://gnome.pages.gitlab.gnome.org/gtkmm-documentation/">
  * Programming with gtkmm</a> book for a tutorial on programming with gtkmm and
- * glibmm.
+ * glibmm. The online version of this tutorial describes gtkmm-4.0 and glibmm-2.68.
  *
  * @section features Features
  *
- * - Glib::ustring: A UTF-8 string class that can be used interchangably with std::string. Plus @ref StringUtils
+ * - Glib::ustring: A UTF-8 string class that can be used interchangably with std::string.
+ *   Plus @ref StringUtils
  * - Glib::RefPtr: A reference-counting smartpointer, for use with Glib::ObjectBase or similar
  * - @ref CharsetConv
  * - Glib::Regex: Regular expression string matching.
@@ -66,6 +66,21 @@
  * @code
  * g++ program.cc -o program  `pkg-config --cflags --libs glibmm-2.4 giomm-2.4`
  * @endcode
+ * If your version of g++ is not C++11-compliant by default,
+ * add the @c -std=c++11 option.
+ *
+ * If you use <a href="https://mesonbuild.com/">Meson</a>, include the following
+ * in @c meson.build:
+ * @code
+ * glibmm_dep = dependency('glibmm-2.4')
+ * giomm_dep = dependency('giomm-2.4')
+ * program_name = 'program'
+ * cpp_sources = [ 'program.cc' ]
+ * executable(program_name,
+ *   cpp_sources,
+ *   dependencies: [ glibmm_dep, giomm_dep ]
+ * )
+ * @endcode
  *
  * Alternatively, if using autoconf, use the following in @c configure.ac:
  * @code
@@ -80,16 +95,22 @@
  */
 
 #include <glibmmconfig.h>
-//#include <glibmm/i18n.h> //This must be included by the application, after system headers such as <iostream>.
+//#include <glibmm/i18n.h> //This must be included by the application, after system headers such as
+//<iostream>.
 
-//Include this first because we need it to be the first thing to include <glib.h>, 
-//so we can do an undef trick to still use deprecated API in the header: 
+// Include this first because we need it to be the first thing to include <glib.h>,
+// so we can do an undef trick to still use deprecated API in the header:
 #include <glibmm/thread.h>
 
 #include <glibmm/threads.h>
 
 #include <glibmm/arrayhandle.h>
 #include <glibmm/balancedtree.h>
+#include <glibmm/base64.h>
+#ifndef GLIBMM_INCLUDED_FROM_WRAP_INIT_CC
+// wrap_init.cc includes this file after it has cleared G_GNUC_CONST.
+#include <glibmm/binding.h>
+#endif
 #include <glibmm/bytearray.h>
 #include <glibmm/bytes.h>
 #include <glibmm/checksum.h>
@@ -97,7 +118,9 @@
 #include <glibmm/containerhandle_shared.h>
 #include <glibmm/convert.h>
 #include <glibmm/date.h>
+#include <glibmm/datetime.h>
 #include <glibmm/dispatcher.h>
+#include <glibmm/enums.h>
 #include <glibmm/error.h>
 #include <glibmm/exception.h>
 #include <glibmm/exceptionhandler.h>
@@ -134,14 +157,17 @@
 #include <glibmm/threadpool.h>
 #include <glibmm/timer.h>
 #include <glibmm/timeval.h>
+#include <glibmm/timezone.h>
 #include <glibmm/uriutils.h>
 #include <glibmm/ustring.h>
 #include <glibmm/value.h>
 #include <glibmm/valuearray.h>
 #include <glibmm/variant.h>
+#include <glibmm/variantdict.h>
 #include <glibmm/variantiter.h>
 #include <glibmm/varianttype.h>
 #include <glibmm/vectorutils.h>
+#include <glibmm/weakref.h>
 #include <glibmm/wrap.h>
 
 #endif /* _GLIBMM_H */
