@@ -1,7 +1,5 @@
-// -*- c++ -*-
 #ifndef _GLIBMM_EXCEPTION_H
 #define _GLIBMM_EXCEPTION_H
-/* $Id$ */
 
 /* exception.h
  *
@@ -18,25 +16,21 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free
- * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * License along with this library.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <glibmm/ustring.h>
 
-
 namespace Glib
 {
 
-class Exception
+class GLIBMM_API Exception
 {
 public:
-  virtual ~Exception() throw() = 0;
+  virtual ~Exception() noexcept = 0;
   virtual Glib::ustring what() const = 0;
 };
 
 } // namespace Glib
 
-
 #endif /* _GLIBMM_EXCEPTION_H */
-
