@@ -1,4 +1,3 @@
-// -*- c++ -*-
 #ifndef _GLIBMM_EXCEPTIONHANDLER_H
 #define _GLIBMM_EXCEPTIONHANDLER_H
 
@@ -17,27 +16,26 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free
- * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * License along with this library.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <glibmmconfig.h>
 #include <sigc++/sigc++.h>
 
-
 namespace Glib
 {
 
 /** Specify a slot to be called when an exception is thrown by a signal handler.
  */
+GLIBMM_API
 sigc::connection add_exception_handler(const sigc::slot<void>& slot);
 
 #ifndef DOXYGEN_SHOULD_SKIP_THIS
 // internal
-void exception_handlers_invoke() throw();
-#endif //DOXYGEN_SHOULD_SKIP_THIS
+GLIBMM_API
+void exception_handlers_invoke() noexcept;
+#endif // DOXYGEN_SHOULD_SKIP_THIS
 
 } // namespace Glib
 
 #endif /* _GLIBMM_EXCEPTIONHANDLER_H */
-
