@@ -1,35 +1,33 @@
-// -*- c++ -*-
 #ifndef _GLIBMM_SIGNALPROXY_CONNECTIONNODE_H
 #define _GLIBMM_SIGNALPROXY_CONNECTIONNODE_H
 
-/* $Id: signalproxy_connectionnode.h 445 2007-09-25 14:02:47Z murrayc $ */
-
 /* signalproxy_connectionnode.h
  *
  * Copyright (C) 2002 The gtkmm Development Team
  *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
+ * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 2.1 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the Free
- * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+#include <glibmmconfig.h>
+
 #include <sigc++/sigc++.h>
-#include <glib/gtypes.h>
+#include <glib.h>
 
 #ifndef DOXYGEN_SHOULD_SKIP_THIS
-typedef struct _GObject GObject;
-typedef struct _GClosure GClosure;
-#endif //DOXYGEN_SHOULD_SKIP_THIS
+using GObject = struct _GObject;
+using GClosure = struct _GClosure;
+#endif // DOXYGEN_SHOULD_SKIP_THIS
 
 namespace Glib
 {
@@ -40,15 +38,21 @@ namespace Glib
   * It lives between the layer of Gtk+ and libsigc++.
   * It is very much an internal class.
   */
-class SignalProxyConnectionNode
+class GLIBMM_API SignalProxyConnectionNode
 {
 public:
-
   /** @param slot The signal handler for the glib signal.
    *  @param gobject The GObject that might emit this glib signal
    */
   SignalProxyConnectionNode(const sigc::slot_base& slot, GObject* gobject);
 
+  /** @param slot The signal handler for the glib signal.
+   *  @param gobject The GObject that might emit this glib signal
+   *
+   * @newin{2,48}
+   */
+  SignalProxyConnectionNode(sigc::slot_base&& slot, GObject* gobject);
+
   /** Callback that is executed when the slot becomes invalid.
    * This callback is registered in the slot.
    * @param data The SignalProxyConnectionNode object (@p this).
@@ -68,10 +72,8 @@ protected:
   GObject* object_;
 };
 
-#endif //DOXYGEN_SHOULD_SKIP_THIS
+#endif // DOXYGEN_SHOULD_SKIP_THIS
 
 } /* namespace Glib */
 
-
 #endif /* _GLIBMM_SIGNALPROXY_CONNECTIONNODE_H */
-
